@@ -67,7 +67,8 @@ static void discard_vcc(VCC *vcc)
 {
     if (do_close(vcc->fd))
 	diag(COMPONENT,DIAG_ERROR,"close: %s",strerror(errno));
-    Q_REMOVE(vcc->entry->vccs,vcc);
+    if (vcc->entry) Q_REMOVE(vcc->entry->vccs,vcc);
+    else Q_REMOVE(unidirectional_vccs,vcc);
     free(vcc);
 }
 
@@ -85,6 +86,8 @@ void discard_vccs(ENTRY *entry)
 
 void discard_entry(ENTRY *entry)
 {
+    if (entry->notify)
+	diag(COMPONENT,DIAG_FATAL,"discard_entry: entry %p had notifications");
     STOP_TIMER(entry);
     discard_vccs(entry);
     if (!entry->itf) Q_REMOVE(unknown_incoming,entry);
@@ -105,7 +108,7 @@ void discard_entry(ENTRY *entry)
  */
 
 
-static int want_arp_srv(ITF *itf)
+static int want_arp_srv(const ITF *itf)
 {
     VCC *vcc;
     int fd;
@@ -238,7 +241,7 @@ static void arp_request(ITF *itf,uint32_t ip)
 }
 
 
-static void inarp_for_itf(VCC *vcc,ITF *itf)
+static void inarp_for_itf(const VCC *vcc,const ITF *itf)
 {
     if (itf->local_ip) {
 	diag(COMPONENT,DIAG_DEBUG,"  for itf %d",itf->number);
@@ -297,6 +300,18 @@ static void inarp_reply(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 }
 
 
+static void send_notifications(ENTRY *entry,int success)
+{
+    NOTIFY *this;
+
+    while ((this = entry->notify)) {
+	entry->notify = this->next;
+	notify(&this->ctx,entry->ip,success ? entry : NULL);
+	free(this);
+    }
+}
+
+
 static void timer_expiration(void *user);
 
 
@@ -341,6 +356,7 @@ static void timeout(ENTRY *entry)
     entry->timer = NULL;
     switch (entry->state) {
 	case as_resolv:
+	    send_notifications(entry,0);
 	    if ((entry->flags & ATF_ARPSRV) && !entry->vccs) {
 		if (entry->itf) want_arp_srv(entry->itf);
 		break;
@@ -473,6 +489,7 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 	free(vcc->entry);
 	vcc->entry = entry;
 	Q_INSERT_HEAD(entry->vccs,vcc);
+	entry->flags &= ~ATF_NOVC;
 	assert(!vcc->connecting);
 	if (set_ip(vcc->fd,ip) < 0)
 	    diag(COMPONENT,DIAG_ERROR,"set_ip: %s",strerror(errno));
@@ -509,6 +526,7 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
       && atm_equal((struct sockaddr *) entry->addr,(struct sockaddr *) addr,0,
       0)))) return; /* no news */
     STOP_TIMER(entry);
+    if (entry->ip != ip) send_notifications(entry,0);
     entry->ip = ip;
     if (!entry->itf) {
 	entry->itf = itf;
@@ -543,8 +561,11 @@ static void learn(VCC *vcc,uint32_t ip,struct sockaddr_atmsvc *addr)
 	if (!walk->connecting)
 	    if (set_ip(walk->fd,ip) < 0)
 		diag(COMPONENT,DIAG_ERROR,"set_ip: %s",strerror(errno));
-    if (entry->state != as_valid && !entry->vccs && itf->arp_srv)
-	connect_me(entry);
+    if (entry->state != as_valid) {
+	if (!entry->vccs && itf->arp_srv && !(entry->flags & ATF_NOVC))
+	    connect_me(entry);
+	send_notifications(entry,1);
+    }
     if ((entry->flags & ATF_ARPSRV) || !(entry->flags & ATF_PERM))
 	if (entry->itf->arp_srv) START_TIMER(entry,CREVAL);
 	else START_TIMER(entry,SREVAL);
@@ -562,12 +583,55 @@ static void learn_nak(uint32_t ip)
     if (!itf) return;
     entry = lookup_ip(itf,ip);
     if (!entry || entry->state != as_resolv) return;
+    send_notifications(entry,0);
     if (entry->flags & ATF_PERM) return;
-    if (entry->vccs || (entry->flags & ATF_PERM)) entry->state = as_invalid;
+    if (entry->vccs) entry->state = as_invalid;
     else discard_entry(entry);
 }
 
 
+/*
+ * Returns:
+ *  <0  resolution is not possible (*entry may be clobbered)
+ *   0  resolution has succeeded (entry in *entry)
+ *  >0  resolution is proceeding (entry in *entry)
+ */
+
+static int resolve(ITF *itf,uint32_t ip,ENTRY **entry,int want_vc)
+{
+    *entry = lookup_ip(itf,ip);
+    if ((!*entry || (*entry)->state != as_valid) && !itf->arp_srv)
+	return -1; /* bad luck - no ARP server when we need one */
+    if (*entry) {
+	if (want_vc) (*entry)->flags &= ~ATF_NOVC;
+	switch ((*entry)->state) {
+	    case as_resolv:
+		return 1; /* somebody else is already taking care of that */
+	    case as_valid:
+		if (!(*entry)->vccs && !((*entry)->flags & ATF_NOVC))
+		    connect_me(*entry);
+		return 0;
+	    case as_invalid:
+		if ((*entry)->svc && (*entry)->itf && (*entry)->itf->arp_srv &&
+		    !((*entry)->flags & ATF_ARPSRV)) break;
+		return -1;
+	    default:
+		diag(COMPONENT,DIAG_FATAL,"bad state %d",(*entry)->state);
+	}
+    }
+    else {
+	*entry = alloc_entry(1);
+	(*entry)->flags = ATF_PUBL | (want_vc ? 0 : ATF_NOVC);
+	(*entry)->ip = ip;
+	(*entry)->itf = itf;
+	Q_INSERT_HEAD(itf->table,*entry);
+	(*entry)->qos = itf->qos;
+    }
+    revalidate(*entry);
+    return 1;
+}
+
+
 void need_ip(int itf_num,uint32_t ip)
 {
     ITF *itf;
@@ -580,32 +644,40 @@ void need_ip(int itf_num,uint32_t ip)
 	diag(COMPONENT,DIAG_ERROR,"itf %d not found",itf_num);
 	return;
     }
-    entry = lookup_ip(itf,ip);
-    if ((!entry || entry->state != as_valid) && !itf->arp_srv)
-	return; /* bad luck - no ARP server when we need one */
-    if (entry)
-	switch (entry->state) {
-	    case as_resolv:
-		return; /* somebody else is already taking care of that */
-	    case as_valid:
-		if (!entry->vccs) connect_me(entry);
-		return;
-	    case as_invalid:
-		if (entry->svc && entry->itf && entry->itf->arp_srv &&
-		    !(entry->flags & ATF_ARPSRV)) break;
-		return;
-	    default:
-		diag(COMPONENT,DIAG_FATAL,"bad state %d",entry->state);
-	}
-    else {
-	entry = alloc_entry(1);
-	entry->flags = ATF_PUBL;
-	entry->ip = ip;
-	entry->itf = itf;
-	Q_INSERT_HEAD(itf->table,entry);
-	entry->qos = itf->qos;
+    (void) resolve(itf,ip,&entry,1);
+}
+
+
+void query_ip(const UN_CTX *ctx,uint32_t ip)
+{
+    ITF *itf;
+    ENTRY *entry;
+    NOTIFY *notifier;
+    int result;
+
+    diag(COMPONENT,DIAG_DEBUG,"query for %d.%d.%d.%d",
+      ((unsigned char *) &ip)[0],((unsigned char *) &ip)[1],
+      ((unsigned char *) &ip)[2],((unsigned char *) &ip)[3]);
+    if (!(itf = lookup_itf_by_ip(ip))) {
+	diag(COMPONENT,DIAG_WARN,"itf for %d.%d.%d.%d not found",
+	    ((unsigned char *) &ip)[0],((unsigned char *) &ip)[1],
+	    ((unsigned char *) &ip)[2],((unsigned char *) &ip)[3]);
+	notify(ctx,ip,NULL);
+	return;
     }
-    revalidate(entry);
+    result = resolve(itf,ip,&entry,0);
+    if (result < 0) {
+	notify(ctx,ip,NULL);
+	return;
+    }
+    if (!result) {
+	notify(ctx,ip,entry);
+	return;
+    }
+    notifier = alloc_t(NOTIFY);
+    notifier->ctx = *ctx;
+    notifier->next = entry->notify;
+    entry->notify = notifier;
 }
 
 
@@ -819,10 +891,12 @@ static int ioctl_delete(ITF *itf,uint32_t ip,int flags)
 	return -ENOENT;
     }
     if ((flags ^ entry->flags) & ATF_ARPSRV) return -EINVAL;
+    send_notifications(entry,0);
     if ((entry->flags & ATF_ARPSRV) && entry->itf)
 	for (walk = entry->itf->table; walk; walk = next) {
 	    next = walk->next;
-	    if (walk != entry && walk->state == as_resolv)
+	    if (walk != entry && walk->state == as_resolv) {
+		send_notifications(walk,0);
 		if (!walk->vccs && !(walk->flags & ATF_PERM))
 		   /* PERM is rather unlikely here, since this would be a
 		      second ARP server (only ARP servers can go as_resolv if
@@ -832,6 +906,7 @@ static int ioctl_delete(ITF *itf,uint32_t ip,int flags)
 		    STOP_TIMER(walk);
 		    walk->state = as_invalid;
 		}
+	    }
 	}
     discard_entry(entry);
     return 0;
@@ -954,3 +1029,10 @@ void incoming_call(VCC *vcc)
     START_TIMER(vcc->entry,REPLY);
     inarp_request(vcc->entry);
 }
+
+
+void incoming_unidirectional(VCC *vcc)
+{
+    diag(COMPONENT,DIAG_DEBUG,"incoming unidirectional VCC 0x%p",vcc);
+    /* don't put it into ATMARP table */
+}
