@@ -31,14 +31,14 @@ AsnOidResult AsnOidCompare(AsnOid *o1, AsnOid *o2)
   if(o1->octetLen > o2->octetLen)
   {
     result = memcmp(o1->octs, o2->octs, o2->octetLen);
-    if(result == -1)
+    if(result < 0)
       return AsnOidLess;
     else
       return AsnOidGreater;
   } else if(o1->octetLen < o2->octetLen)
   {
     result = memcmp(o1->octs, o2->octs, o1->octetLen);
-    if(result == 1)
+    if(result > 0)
       return AsnOidGreater;
     else if(result == 0)
       return AsnOidRoot;
@@ -47,7 +47,7 @@ AsnOidResult AsnOidCompare(AsnOid *o1, AsnOid *o2)
   } else
   {
     result = memcmp(o1->octs, o2->octs, o1->octetLen);
-    if(result == -1)
+    if(result < 0)
       return AsnOidLess;
     else if(result == 0)
       return AsnOidEqual;
