@@ -19,7 +19,7 @@
 #include <sys/time.h>
 
 /* ATM includes */
-#include <linux/atm.h>
+#include <atm.h>
 
 /* Local includes */
 #include "lecs_load.h"
