@@ -36,7 +36,7 @@
 #include <string.h>
 #include <stdlib.h>  /* for malloc() and free() */
 /* Atm includes */
-#include <linux/atm.h>
+#include <atm.h>
 #include <linux/atmlec.h>
 
 /* Local includes */
