@@ -1,6 +1,6 @@
 /* uni.h - Various Q.2931/Q.2971/Q.2963.1/UNI 3.x/UNI 4.0 constants */
 
-/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
 
 /*
  * Note: some values don't appear in UNI 3.0 or 3.1 but are taken from Q.2931
@@ -130,9 +130,9 @@
 #define ATM_CV_CI_FAIL		 36 /* VPCI/VCI assignment failure */
 #define ATM_CV_UCR_UNAVAIL	 37 /* user cell rate not available */
 #endif
-#define ATM_CV_CI_NET_OOO	 38 /* network out of order - unused */
-#define ATM_CV_CI_TEMP_FAIL	 41 /* temporary failure */
-#define ATM_CV_CI_ACC_INF_DISC	 43 /* access information discarded */
+#define ATM_CV_NET_OOO		 38 /* network out of order - unused */
+#define ATM_CV_TEMP_FAIL	 41 /* temporary failure */
+#define ATM_CV_ACC_INF_DISC	 43 /* access information discarded */
 #define ATM_CV_NO_CI		 45 /* no VPCI/VCI available */
 #define ATM_CV_RES_UNAVAIL	 47 /* resource unavailable, unspecified */
 /* ----------------------------------- service or option not available */
